<?php
/**
 * @file
 * Contains support form for miniOrange 2FA Login
 *     Module.
 */

namespace Drupal\miniorange_2fa\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\user\Entity\User;
use stdClass;
use Drupal\Core\Url;

class MoAuthUserManagement extends FormBase
{
    public function getFormId()
    {
        return 'miniorange_2fa_user_management';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        global $base_url;
        $utilities = new MoAuthUtilities();

        $form['markup_top_2'] = array(
            '#markup' => '<div class="mo_2fa_table_layout_1"><div class="mo_2fa_table_layout mo_2fa_container">'
        );

        $disabled = False;
        if (!$utilities::isCustomerRegistered()) {
            $form['header'] = array(
                '#markup' => t('<div class="mo_2fa_register_message"><p>' . t('You need to') . ' <a href="' . $base_url . '/admin/config/people/miniorange_2fa/customer_setup">' . t('Register/Login') . '</a> ' . t('with miniOrange before using this module.') . '</p></div>'),
            );
            $disabled = True;
        }

        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "miniorange_2fa/miniorange_2fa.admin",
                    "miniorange_2fa/miniorange_2fa.license",
                )
            ),
        );

        /**
         * Create container to hold all the form elements.
         */
        $form['mo_user_management'] = array(
            '#type' => 'fieldset',
            '#title' => t('User Management'),
            '#attributes' => array('style' => 'padding:0% 2% 17%')
        );

        $form['mo_user_management']['starting_markup'] = array(
          '#markup' => '<br><hr><br><div class="mo_2fa_highlight_background_note"><strong>' . t('Note:') . ' </strong>' . t('If you want to reset or disable the 2FA for any user, you can do it from this section.') . ' <strong>' . t('If you reset the 2FA for any user, then that user has to go through the inline registration process to setup the 2FA again.') . '</strong></div><br>',
        );

        $form['mo_user_management']['mo_auth_username_to_reset_2fa'] = array(
            '#type' => 'entity_autocomplete',
            '#target_type' => 'user',
            '#disabled' => $disabled,
            '#attributes' => array(
                'placeholder' => $this->t('Enter username'),
            ),
            '#prefix' => '<div class="container-inline">',
        );

        $connection = \Drupal::database();
        $query      = $connection->query("SELECT * FROM {UserAuthenticationType}");
        $result     = $query->fetchAll();

        $form['mo_user_management']['submit_user_form'] = array(
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => $this->t('Get info'),
            '#submit' => ['::getInfo'],
            '#suffix' => '</div>',
            '#disabled' => $disabled,
            '#ajax' => array(
                'callback' => '::promptCallback',
                'wrapper' => 'replace-textfield-container',
                'event' => 'click',
            ),
        );

        $form['mo_user_management']['container'] = array(
            '#type' => 'container',
            '#attributes' => ['id' => 'replace-textfield-container'],
        );

        if($form_state->getValue('mo_auth_username_to_reset_2fa') !== NULL) {
            $state_user     = $form_state->getValue('mo_auth_username_to_reset_2fa');
            $is_2fa_enabled = false;
            $data           = new stdClass();

            foreach ($result as $key) {
                if ($key->uid == $state_user) {
                    $is_2fa_enabled = true;
                    $data = $key;
                }
            }

            if ($is_2fa_enabled) {
                $row    = array();
                $roles  = User::load($data->uid)->getRoles();
                $roles  = implode(", ", $roles);
                $status = $data->enabled ? 'Enabled' : 'Disabled';
                $phone  = empty($data->phone_number) ? '-' : $data->phone_number;

                $row    = array(
                  array($this->t('User ID'), $data->uid),
                  array($this->t('Username'), User::load($data->uid)->getAccountName()),
                  array($this->t('User Email'), $data->miniorange_registered_email),
                  array($this->t('Phone Number'), $phone),
                  array($this->t('User Roles'), $roles),
                  array($this->t('2FA Method'), $data->activated_auth_methods),
                  array($this->t('2FA Status'), $this->t($status)),
                );

                $form['mo_user_management']['container']['userinfo'] = array(
                    '#type' => 'table',
                    '#header' => array($this->t('Attribute'), $this->t('Value')),
                    '#title' => $this->t('User info table'),
                    '#rows' => $row,
                );

                $form['mo_user_management']['container']['userinfo'][7]['attribute'] = array(
                    '#markup' => 'Action',
                );

                $form['mo_user_management']['container']['userinfo'][7]['value'] = array(
                    '#type' => 'dropbutton',
                    '#dropbutton_type' => 'small',
                    '#links' => array(
                        'disable' => array(
                            'title' => !$data->enabled ? $this->t('Enable' ): $this->t('Disable'),
                             'url'  => Url::fromRoute('miniorange_2fa.changes_2fa_status', array('user' => $data->uid)),
                        ),
                        'reset' => array(
                            'title' => $this->t('Reset'),
                            'url' => Url::fromRoute('miniorange_2fa.rest_2fa', array('user' => $data->uid )),
                        ),
                    ),
                );



            }
            else {
                $form['mo_user_management']['container']['userinfo'] = array(
                    '#markup' => $this->t('2FA is not configured for this user'),
                );
            }
        }

        $form['mo_user_management']['end'] = array(
            '#markup' => '<br><br><br></div>',
        );

        $utilities::miniOrange_advertise_network_security($form, $form_state);

        return $form;
    }

    /**
     * @param $form
     * @param FormStateInterface $form_state
     * @return mixed
     */
    public function promptCallback($form, FormStateInterface $form_state) {
        return $form['mo_user_management']['container'];
    }

    /**
     * @param $form
     * @param FormStateInterface $form_state
     * @return void
     */
    public function getInfo($form, FormStateInterface $form_state) {
        $form_state->setRebuild();
    }

    public function submitForm(array &$form, FormStateInterface $form_state)
    {

    }
}
