<?php

namespace Drupal\miniorange_2fa\Form;

use Drupal\Core\Ajax\MessageCommand;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\miniorange_2fa\MiniorangeUser;
use Drupal\miniorange_2fa\MoAuthUtilities;
use Drupal\miniorange_2fa\UsersAPIHandler;
use Drupal\miniorange_2fa\AuthenticationType;
use Drupal\miniorange_2fa\AuthenticationAPIHandler;
use Drupal\miniorange_2fa\MiniorangeCustomerProfile;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * @file
 *  This is used to authenticate user during
 *     login.
 */
class miniorange_select_method extends FormBase
{
    public function getFormId()
    {
        return 'mo_auth_select_method';
    }

    public function buildForm(array $form, FormStateInterface $form_state)
    {
        $utilities = new MoAuthUtilities();
        $session = $utilities->getSession();
        $moMfaSession = $session->get("mo_auth", null);

        if (is_null($moMfaSession) || !isset($moMfaSession['uid']) || !isset($moMfaSession['status']) || $moMfaSession['status'] !== '1ST_FACTOR_AUTHENTICATED' || $moMfaSession['challenge'] !== 1) {
            $session->remove('mo_auth');
            $utilities->mo_add_loggers_for_failures('Error at ' . __FILE__ . ' Function: ' . __FUNCTION__ . ' Line number: ' . __LINE__, 'error');
            $message = 'Something went wrong. Please try again.';
            $utilities->redirectUserToLoginPage($message);
        }

        $url_parts = $utilities->mo_auth_get_url_parts();
        end($url_parts);
        $user_id = prev($url_parts);
        if ($moMfaSession['uid'] != $user_id) {
            $session->remove('mo_auth');
            $utilities->mo_add_loggers_for_failures('URL change detected', 'error');
            $message = 'Something went wrong. Please try again.';
            $utilities->redirectUserToLoginPage($message);
        }

        $form['markup_library'] = array(
            '#attached' => [
                'library' => [
                    "miniorange_2fa/miniorange_2fa.admin",
                    "miniorange_2fa/miniorange_2fa.license",
                ]
            ],
        );

        $options = $utilities->get_2fa_methods_for_inline_registration(TRUE);

        $form['mo_auth_method'] = array(
            '#type' => 'select',
            '#default_value' => array_keys($options)[0],
            '#options' => $options,
            '#required' => TRUE,
        );

        $phoneNumber = $utilities->getUserPhoneNumber($moMfaSession['uid']);

        $form['mo_auth_phone_number'] = array(
            '#type' => 'textfield',
            '#title' => t('Phone Number'),
            '#id' => 'query_phone',
            '#description' => t('Enter number with country code Eg. +00xxxxxxxxxx'),
            '#states' => array(
                'visible' => array(
                    ':input[name = "mo_auth_method"]' => [
                        ['value' => 'SMS'],
                        ['value' => 'SMS AND EMAIL'],
                        ['value' => 'PHONE VERIFICATION'],
                    ]
                ),
            ),
            '#attributes' => array(
                'class' => [
                    'query_phone',
                    'mo2f-textbox',
                    'mo2f-textbox-otp'
                ],
            ),
        );

        if (!is_null($phoneNumber)) {
            $form['mo_auth_phone_number']['#default_value'] = $phoneNumber;
        }

        $form['actions']['#type'] = 'actions';
        $form['actions']['login'] = [
            '#type' => 'submit',
            '#button_type' => 'primary',
            '#value' => t('Next'),
        ];

        $form['actions']['cancel'] = array(
            '#type' => 'submit',
            '#value' => t('Cancel'),
            '#button_type' => 'danger',
            '#submit' => array('::moCancelInlineRegistration'),
            '#limit_validation_errors' => array(), //skip the required field validation
        );

        if ($utilities->isSkipNotAllowed()) {
            $form['actions']['skip_mfa'] = array(
                '#type' => 'submit',
                '#value' => t('Skip'),
                '#submit' => array('::handle_skip_mfa'),
                '#limit_validation_errors' => array(), //skip the required field validation
            );
        }

        return $form;
    }

    public function validateForm(array &$form, FormStateInterface $form_state)
    {
        $form_values = $form_state->getValues();
        $authMethod = $form_values['mo_auth_method'];
        $phoneNumber = str_replace(' ', '', $form_values['mo_auth_phone_number']);
        if ($authMethod === 'SMS' || $authMethod === 'SMS AND EMAIL' || $authMethod === 'PHONE VERIFICATION') {
            if (!preg_match("^[\+]?[0-9]{1,4}\s?[0-9]{7,12}^", $phoneNumber)) {
                $form_state->setErrorByName('mo_auth_phone_number', $this->t('Please match the requested format'));
            }
        }
    }

    public function submitForm(array &$form, FormStateInterface $form_state)
    {
        $utilities = new MoAuthUtilities();
        $form_values = $form_state->getValues();
        $authMethod = $form_values['mo_auth_method'];
        $session = $utilities->getSession();
        $moMfaSession = $session->get("mo_auth", null);

        $account = User::load($moMfaSession['uid']);
        $email = $account->get('mail')->value;

        $customer = new MiniorangeCustomerProfile();
        $auth_api_handler = new AuthenticationAPIHandler($customer->getCustomerID(), $customer->getAPIKey());

        /**
         * Create user
         */
        self::moCreateUser($email);

        if (in_array($authMethod, $utilities->mo_TOTP_2fa_mentods())) {
            $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $email, NULL, NULL, NULL);
            $response = $auth_api_handler->getGoogleAuthSecret($miniorange_user);
            if (is_object($response) && $response->status == 'SUCCESS') {
                $moMfaSession['mo_challenge_response'] = $response;
                $moMfaSession['authentication_method'] = $authMethod;
                $session->set('mo_auth', $moMfaSession);
                $session->save();
            } else {
                $utilities->mo_add_loggers_for_failures(is_object($response)?$response->message:'An unexpected error occured.', 'error');
                $utilities->redirectUserToLoginPage('An error occurred while processing your request. Please try again after sometime.');
            }
        } elseif ($authMethod == AuthenticationType::$QR_CODE['code'] || $authMethod == AuthenticationType::$SOFT_TOKEN['code'] || $authMethod == AuthenticationType::$PUSH_NOTIFICATIONS['code']) {
            $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $email, NULL, NULL, NULL);
            $response = $auth_api_handler->register($miniorange_user, AuthenticationType::$QR_CODE['code'], NULL, NULL, NULL);
            if (is_object($response) && $response->status == 'IN_PROGRESS') {
                $moMfaSession['mo_challenge_response'] = $response;
                $moMfaSession['authentication_method'] = $authMethod;
                $session->set('mo_auth', $moMfaSession);
                $session->save();
            } else {
                $utilities->mo_add_loggers_for_failures(is_object($response)?$response->message:'An unexpected error occured.', 'error');
                $utilities->redirectUserToLoginPage('An error occurred while processing your request. Please try again after sometime.');
            }
        } elseif ($authMethod == AuthenticationType::$SMS['code'] || $authMethod == AuthenticationType::$EMAIL['code'] || $authMethod == AuthenticationType::$SMS_AND_EMAIL['code'] || $authMethod == AuthenticationType::$OTP_OVER_PHONE['code'] || $authMethod == AuthenticationType::$EMAIL_VERIFICATION['code']) {
            $phoneNumber = str_replace(' ', '', $form_values['mo_auth_phone_number']);
            if ($authMethod == AuthenticationType::$SMS_AND_EMAIL['code']) {
                $miniorange_user = new MiniorangeUser($customer->getCustomerID(), NULL, $phoneNumber, NULL, $authMethod, $email);
            } elseif ($authMethod == AuthenticationType::$EMAIL['code'] || $authMethod == AuthenticationType::$EMAIL_VERIFICATION['code']) {
                $miniorange_user = new MiniorangeUser($customer->getCustomerID(), NULL, NULL, NULL, $authMethod, $email);
            } elseif ($authMethod == AuthenticationType::$SMS['code'] || $authMethod == AuthenticationType::$OTP_OVER_PHONE['code']) {
                $miniorange_user = new MiniorangeUser($customer->getCustomerID(), NULL, $phoneNumber, NULL, $authMethod, NULL);
            }
            $response = $auth_api_handler->challenge($miniorange_user);

            if (is_object($response) && $response->status == 'SUCCESS') {
                \Drupal::messenger()->addStatus(t($response->message));
                $moMfaSession['mo_challenge_response'] = $response;
                $moMfaSession['authentication_method'] = $authMethod;
                $moMfaSession['phone_number'] = $phoneNumber;
                $session->set('mo_auth', $moMfaSession);
                $session->save();
            } else {
                $user_api_handler = new UsersAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
                $response = $user_api_handler->fetchLicense();
                /**
                 * Delete this if block and fetchLicense_drupal8 function once all the Drupal 8 2FA customers shift on the Drupal 2FA plan.
                 */
                if (is_object($response) && $response->status == 'FAILED') {
                    $response = $user_api_handler->fetchLicense_drupal8();
                }

                if (is_object($response) && ($response->smsRemaining == 0 || $response->emailRemaining == 0 || $response->ivrRemaining == 0)) {
                    $utilities->mo_add_loggers_for_failures(t('The number of OTP transactions have exhausted. Please recharge your account with SMS/Email/IVR transactions.'), 'error');
                }
                $utilities->mo_add_loggers_for_failures(is_object($response)?$response->message:'An unexpected error occured.', 'error');
                $utilities->redirectUserToLoginPage('An error occurred while processing your request. Please try again after sometime.');
            }
        } elseif ($authMethod == AuthenticationType::$KBA['code']) {
            $message = 'Please choose your security questions (KBA) and answer those:';
            \Drupal::messenger()->addStatus(t($message));
            $moMfaSession['authentication_method'] = $authMethod;
            $moMfaSession['mo_challenge_response'] = '';
            $session->set('mo_auth', $moMfaSession);
            $session->save();
        } elseif ($authMethod == AuthenticationType::$HARDWARE_TOKEN['code']) {
            $message = 'Insert your Hardware Token and touch the Hardware Token button to enter OTP';
            \Drupal::messenger()->addStatus(t($message));
            $moMfaSession['authentication_method'] = $authMethod;
            $moMfaSession['mo_challenge_response'] = '';
            $session->set('mo_auth', $moMfaSession);
            $session->save();
        }

        $url = Url::fromRoute('miniorange_2fa.configure_method', ['user' => $moMfaSession['uid']])->toString();
        $response = new RedirectResponse($url);
        $response->send();
        exit;
    }

    public function moCreateUser($email)
    {
        $utilities = new MoAuthUtilities();
        $customer = new MiniorangeCustomerProfile();
        $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $email, NULL, NULL, AuthenticationType::$EMAIL['code']);
        $user_api_handler = new UsersAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
        $response = $user_api_handler->search($miniorange_user);

        if (is_object($response) && $response->status == 'USER_NOT_FOUND') { //$response->status == 'USER_FOUND'
            //Create user in miniOrange dashboard
            $createResponse = $user_api_handler->create($miniorange_user);
            if (is_object($createResponse) && isset($createResponse->status) && $createResponse->status == 'SUCCESS') {
                $variables_and_values = array('mo_user_limit_exceed');
                $utilities->miniOrange_set_get_configurations($variables_and_values, 'CLEAR');
                //Update User Auth method to OTP Over EMAIL
                $updateResponse = $user_api_handler->update($miniorange_user);
                if (is_object($updateResponse) && $updateResponse->status != 'SUCCESS') {
                    $utilities->mo_add_loggers_for_failures($updateResponse->message, 'error');
                    $utilities->redirectUserToLoginPage('An error occurred while updating 2FA. Please contact your administrator.');
                }
            } elseif (is_object($createResponse) && isset($createResponse->status) && $createResponse->status == 'ERROR' && $createResponse->message == t('Your user creation limit has been completed. Please upgrade your license to add more users.')) {
                //Check if user creation limit is exceeded
                $variables_and_values = array('mo_user_limit_exceed' => TRUE);
                $utilities->miniOrange_set_get_configurations($variables_and_values, 'SET');
                $utilities->mo_add_loggers_for_failures($createResponse->message, 'error');
                $utilities->redirectUserToLoginPage('An error occurred while configuring 2FA. Please contact your administrator.');
            } else {
                $utilities->mo_add_loggers_for_failures($createResponse->message, 'error');
                $utilities->redirectUserToLoginPage('An error occurred while processing your request. Please contact your administrator.');
            }
        }
    }

    public function moCancelInlineRegistration($message = NULL)
    {
        $session = MoAuthUtilities::getSession();
        $session->remove('mo_auth');
        $url = Url::fromRoute('user.login')->toString();
        $response = new RedirectResponse($url);
        $response->send();
        exit;
    }

    function handle_skip_mfa()
    {
        $utilities = new MoAuthUtilities();
        $session = $utilities->getSession();
        $moMfaSession = $session->get('mo_auth');
        $redirectUrl = Url::fromRoute('user.login')->toString();

        if (isset($moMfaSession['uid'])) {
            $userID = $moMfaSession['uid'];
            $utilities->updateMfaSettingsForUser($userID, 0);

            $account = User::load($userID);
            user_login_finalize($account);

            $this->messenger()->addStatus($this->t("You have successfully disabled 2FA for your account. You can enable it anytime from here."));
            $redirectUrl = Url::fromRoute('miniorange_2fa.user.mo_mfa_form', ['user' => $userID])->toString();
        }
        $response = new RedirectResponse($redirectUrl);
        $response->send();
        exit;
    }
}
